#include "SArandom.h"


struct RANDOM_GENERATOR_GLOBALS	RGG;


void SA_InitRandomGenerator(
/* -W/0D	*/	ul_int	*RandomSeed,
/* R-/0D	*/	FILE	*InStream,
/* -W/0D	*/	FILE	*Verbose)
{
 if(InStream == stdin && Verbose != NULL)
    fprintf(Verbose, "Random generator seed [0-900,000,000]: ");
 fscanf(InStream, "%lu", RandomSeed);
 SA_RMARIN(*RandomSeed);
}

void SA_RMARIN(ul_int IJKL)
{
 ul_int	IJ, KL;
 u_int	I, J, K, L;

 IJ = IJKL / 30082;
 KL = IJKL - 30082 * IJ;
 I  = (IJ / 177) % 177 + 2;
 J  = IJ % 177 + 2;
 K  = (KL / 169) % 178 + 1;
 L  = KL % 169;

 SA_RSTART(I, J, K, L);
}

void SA_RMARIN2(ul_int	IJKL,
		double	*RandomFraction)
{
 SA_RMARIN(IJKL);
 SA_UNI(RandomFraction);
}

void SA_RSTART(u_int I,
	       u_int J,
	       u_int K,
	       u_int L)
{
 u_int	II, JJ,	M;
 double	S, T;

 for(II=0; II<97; II++)	{
    S =	0.0;
    T =	0.5;

    for(JJ=0; JJ<MANTISSA_BITCOUNT; JJ++)	{
      M	= (((I * J) % 179) * K) % 179;
      I	= J;
      J	= K;
      K	= M;
      L	= (53 * L + 1) % 169;

      if ((L*M)%64 >= 32)
	S += T;

      T	*= 0.5;
    }
    RGG.U[II] = S;
 }

/*
 * (double)(1UL << MANTISSA_BITCOUNT) = 16777216.0 :
 */
 RGG.C	= 362436.0   /  (double)(1UL << MANTISSA_BITCOUNT);
 RGG.CD	= 7654321.0  /  (double)(1UL << MANTISSA_BITCOUNT);
 RGG.CM	= 16777213.0 /  (double)(1UL << MANTISSA_BITCOUNT);
 RGG.IP = 97;
 RGG.JP = 33;
}

void SA_UNI(double *Uni)
{
 *Uni = RGG.U [--RGG.IP]  -  RGG.U [--RGG.JP];

 if(*Uni < 0.0)
   *Uni += 1.0;

 RGG.U [RGG.IP]	=  *Uni;

 if(RGG.IP == 0)
   RGG.IP = 97;

 if(RGG.JP == 0)
   RGG.JP = 97;

 RGG.C -= RGG.CD;

 if (RGG.C < 0.0)
   RGG.C += RGG.CM;

 *Uni -= RGG.C;

 if(*Uni < 0.0)
   *Uni += 1.0;
}

void SA_NearStandardGauss6(
/* -W/0D	*/	double	*GaussReal,
/* -W/0D	*/	double	*RandomFraction)
{
	u_int	i;

	*GaussReal	=	0.0;

	for	(i = 0; i < 12; i++)	{
		SA_UNI(RandomFraction);
		*GaussReal += *RandomFraction;
	}
}

void SA_NearStandardGauss0(
/* -W/0D	*/	double	*GaussReal,
/* -W/0D	*/	double	*RandomFraction)
{
	SA_NearStandardGauss6(	GaussReal,
				RandomFraction);

	*GaussReal	-=	6.0;
}

/*****
 * Adapted from:
 * Numerical Recipies in C (1988), p.215	(rejection strategy)
 *
 * Produces two normal distributed numbers (zero mean, unit variance)
*/
void SA_StandardGauss0(
/* -W/0D	*/	double	*GaussReal1,
/* -W/0D	*/	double	*GaussReal2,
/* -W/0D	*/	double	*RandomFraction)
{
	static double	X, Y, R;

	R = 2.0;

	while	(R >= 1.0)	{
		SA_UNI(RandomFraction);
		X = 2.0 * *RandomFraction	- 1.0;
		SA_UNI(RandomFraction);
		Y = 2.0 * *RandomFraction	- 1.0;
		R = X * X + Y * Y;
	}

	R = sqrt(-2.0 * log(R) / R);

	*GaussReal1 = X * R;
	*GaussReal2 = Y * R;
}
