/*
    Title: 	calloc.h
    Purpose:	Matrix allocation functions

    Created:	June 23 1994
    Author:     Adrie Dane <adane@sci.kun.nl>, 
                Frans van der Kloet

    Update:	August 18 1995
                Adrie Dane <adane@sci.kun.nl>, 
*/


/***************
 * HEADER FILE:
*/
#ifdef		__HFILE__
#undef		__HFILE__
#define		__HFILE__	"calloc.h"
#else
#define		__HFILE__	"calloc.h"
#endif

/***************
 * LOCAL STUFF:
*/
#ifndef		CALLOC_H__LOCAL
#define		CALLOC_H__LOCAL

#endif

/***************
 * PUBLIC STUFF:
*/
#ifndef		CALLOC_H__PUBLIC
#define		CALLOC_H__PUBLIC

#include    <stdlib.h>
#include	<stdio.h>
#include	<malloc.h>


#ifdef	IS_ALLOCATED
#undef	IS_ALLOCATED
#endif
#define	IS_ALLOCATED(Pointer)		\
	((Pointer) != NULL)

#ifdef	IS_NOT_ALLOCATED
#undef	IS_NOT_ALLOCATED
#endif
#define	IS_NOT_ALLOCATED(Pointer)	\
	((Pointer) == NULL)

#define ALLOC_ERROR( MacroName, MacroDescript )                             \
do  {                                                                       \
    if (stderr != NULL)						                                \
        fprintf(stderr, "\a%s ERROR\n				                        \
                in macro '%s' defined in file '%s'\n		     		    \
                called in file '%s' on line %u:\n					        \
                \tMemory allocation for dynamic %s failed.\n                \
                \tAborting . . .\n",                                        \
                __HFILE__, MacroName, __HFILE__, __FILE__,                  \
                __LINE__, MacroDescript );                                  \
    exit(1);                                                                \
    } while (0)


#ifdef __cplusplus                                                         
#define TYPEMOD   (void*)
#else
#define TYPEMOD
#endif

#ifdef	CALLOC1D
#undef	CALLOC1D
#endif
#define CALLOC1D(Pointer,L)						                            \
do  {                                                                       \
    TYPEMOD(Pointer) = calloc( (L)+1+1, sizeof(*(Pointer)) );               \
    if ( IS_NOT_ALLOCATED(Pointer) ) 				                        \
        ALLOC_ERROR( "CALLOC1D", "1D-array" );                              \
    (Pointer)++;                                                            \
    } while(0)


#ifdef	CALLOC2D
#undef	CALLOC2D
#endif
#define CALLOC2D(Pointer, K, L)                                             \
do  {                                                                       \
    int	__k__;                                                              \
    CALLOC1D( Pointer, K );                                                 \
    TYPEMOD(*((Pointer)-(1)))  = calloc( (K)*(L+1)+1, sizeof(**(Pointer)) );    \
    if ( IS_NOT_ALLOCATED(*((Pointer)-(1))) )                               \
        ALLOC_ERROR( "CALLOC2D", "2D-array" );                              \
    for (__k__=0;__k__<(K);__k__++)                                         \
        {                                                                   \
        (Pointer)[__k__] = (*((Pointer)-(1)))+(1+__k__*(L+1));              \
        }                                                                   \
    }while(0)



#ifdef	CALLOC3D
#undef	CALLOC3D
#endif
#define CALLOC3D(Pointer, K,L,M)                                            \
do  {                                                                       \
    int __k__,__l__;                                                        \
    CALLOC2D( Pointer, K, L );                                              \
    TYPEMOD(**((Pointer)-(1)))  = calloc( (K)*(L)*(M+1)+1, sizeof(***(Pointer)) );   \
    if ( IS_NOT_ALLOCATED(**((Pointer)-(1))) )                              \
        ALLOC_ERROR( "CALLOC3D", "3D-array" );                              \
    for(__k__=0;__k__<(K);__k__++)                                          \
        for (__l__=0;__l__<(L);__l__++)                                     \
        {                                                                   \
        (Pointer)[__k__][__l__] = (**((Pointer)-(1)))+(1+__k__*((M+1)*(L))+__l__*(M+1));    \
        }                                                                   \
    } while(0)


#ifdef	CALLOC4D
#undef	CALLOC4D
#endif
#define CALLOC4D(Pointer, K,L,M,N)                                          \
do  {                                                                       \
    int __k__,__l__,__m__;                                                  \
    CALLOC3D( Pointer, K, L, M );                                           \
    TYPEMOD(***((Pointer)-(1))) = calloc( (K)*(L)*(M)*(N+1)+1, sizeof(****(Pointer)) );  \
    if ( IS_NOT_ALLOCATED(***((Pointer)-(1))) )                             \
        ALLOC_ERROR( "CALLOC4D", "4D-array" );                              \
    for(__k__=0;__k__<(K);__k__++)                                          \
        for (__l__=0;__l__<(L);__l__++)                                     \
            for (__m__=0;__m__<(M);__m__++)                                 \
            {                                                               \
            (Pointer)[__k__][__l__][__m__] =                                \
                (***((Pointer)-(1)))+(1+__k__*((N+1)*(M)*(L))+__l__*(N+1)*(M)+__m__*(N+1));    \
            }                                                               \
    } while(0)



#ifdef	FREE1D
#undef	FREE1D
#endif
#define FREE1D(Pointer)				                                        \
do  {                                                                       \
    if ( Pointer )      				                                    \
        {                                                                   \
        free( (Pointer-(1)) );				                                \
	    Pointer = (void *)NULL;	        	                                \
        }                                                                   \
    } while(0)


#ifdef	FREE2D
#undef	FREE2D
#endif
#define FREE2D(Pointer)						                                \
do  {                                                                       \
    if ( Pointer )                                                          \
        { 						                                            \
        if ( *(Pointer-(1)) )                                               \
            free( *(Pointer-(1)) );                                         \
        free( (Pointer-(1)) );                                              \
	    Pointer = (void *)NULL;	        	                                \
        }                                                                   \
    } while(0)


#ifdef	FREE3D
#undef	FREE3D
#endif
#define FREE3D(Pointer)						                                \
do  {                                                                       \
    if ( Pointer )                                                          \
        {						                                            \
        if ( *(Pointer-(1))  )                                              \
            {                                                               \
            if ( **(Pointer-(1)) )                                          \
                free( **(Pointer-(1)) );                                    \
            free( *(Pointer-(1)) );                                         \
            }                                                               \
        free( Pointer-(1) );                                                \
	    Pointer = (void *)NULL;	        	                                \
        }                                                                   \
    } while(0)



#ifdef	FREE4D
#undef	FREE4D
#endif
#define FREE4D(Pointer)                                                     \
do  {                                                                       \
    if ( Pointer )                                                          \
        {						                                            \
        if ( *(Pointer-(1))  )                                              \
            {                                                               \
            if ( **(Pointer-(1)) )                                          \
                {                                                           \
                if ( ***(Pointer-(1)) )                                     \
                    free( ***(Pointer-(1)) );                               \
                free( **(Pointer-(1)) );                                    \
                }                                                           \
            free( *(Pointer-(1)) );                                         \
            }                                                               \
        free( Pointer-(1) );                                                \
	    Pointer = (void *)NULL;	        	                                \
        }                                                                   \
    } while(0)


/*****
 * Allocate LOWER triangular part of a square array,
 * diagonal inclusive
*/
#ifdef	CALLOC2D_TRIANG_INCL
#undef	CALLOC2D_TRIANG_INCL
#endif
#define	CALLOC2D_TRIANG_INCL(Pointer, K)	\
do	{								\
	int	__k__;							\
	(Pointer) = calloc(	(K) ,					\
				sizeof(*(Pointer)));			\
	*(Pointer) = calloc(	(K) * ((K) + 1) / 2 ,			\
				sizeof(**(Pointer)));			\
	if	((Pointer)==NULL || *(Pointer)==NULL)	{		\
		if	(stderr != NULL)				\
			fprintf(stderr, "\
\a%sERROR\n\
in macro 'CALLOC2D_TRIANG_INCL' defined in file '%s'\n\
called in file '%s' on line %u:\n\
\tMemory allocation for triangular part of dynamic\n\
\tsquare 2D-array, diagonal inclusive, failed.\n\
\tAborting . . .\n",	__HFILE__, __HFILE__, __FILE__, __LINE__);	\
		exit(1);						\
	}								\
	for(	__k__=1;	__k__<(K);	__k__++)		\
	(Pointer)[__k__]=(Pointer)[__k__-1]+__k__;			\
}	while	(0)


#ifdef	CALLOC2D_TRIANG_EXCL
#undef	CALLOC2D_TRIANG_EXCL
#endif
#define	CALLOC2D_TRIANG_EXCL(Pointer, K)	\
do	{								\
	int	__k__;							\
	(Pointer) = calloc(	(K) ,					\
				sizeof(*(Pointer)));			\
	*(Pointer) = calloc(	(K) * ((K) - 1) / 2 ,			\
				sizeof(**(Pointer)));			\
	if	((Pointer)==NULL || *(Pointer)==NULL)	{		\
		if	(stderr != NULL)				\
			fprintf(stderr, "\
\a%sERROR\n\
in macro 'CALLOC2D_TRIANG_EXCL' defined in file '%s'\n\
called in file '%s' on line %u:\n\
\tMemory allocation for triangular part of dynamic\n\
\tsquare 2D-array, diagonal inclusive, failed.\n\
\tAborting . . .\n",	__HFILE__, __HFILE__, __FILE__, __LINE__);	\
		exit(1);						\
	}								\
	for(	__k__=1;	__k__<(K);	__k__++)		\
	(Pointer)[__k__]=(Pointer)[__k__-1]+__k__-1;			\
}	while	(0)



#ifdef	PRINT1D
#undef	PRINT1D
#endif
#define		PRINT1D(Verbose, FormatString, Pointer, K)\
do	{\
if(Verbose)	{						\
	int __k__;						\
if((Verbose)==stderr || (Verbose)==stdout)			\
fprintf(Verbose,"\n\n(%s)[%s=%d]:\n\n",				\
(#Pointer),  (#K), (K));						\
else								\
fprintf(Verbose,"\n\n(%s)[%s=%d]:\n\n",				\
(#Pointer),  (#K), (K));						\
for(	__k__=0;	__k__<(K);	__k__++)		\
fprintf(Verbose, FormatString, (Pointer)[__k__]);			\
}}while(0)
/* fprintf(Verbose,"\n%d\n\n",(K));				\ */

#ifdef	PRINT2D
#undef	PRINT2D
#endif
#define		PRINT2D(Verbose, FormatString, Pointer, K, L)\
do	{							\
if(Verbose)	{						\
	int __k__, __l__;					\
if((Verbose)==stderr || (Verbose)==stdout)			\
fprintf(Verbose,"\n\n%s[%s=%d][%s=%d]:\n",			\
#Pointer,  (#K), (K), (#L), (L));				\
else								\
fprintf(Verbose,"\n\n%s[%s=%d][%s=%d]:\n",			\
#Pointer,  (#K), (K), (#L), (L));				\
for(	__k__=0;	__k__<(K);	__k__++)	{	\
fprintf(Verbose,"\n");						\
for(	__l__=0;	__l__<(L);	__l__++)	{	\
fprintf(Verbose,FormatString,(Pointer)[__k__][__l__]);		\
}}}								\
}while(0)

/* fprintf(Verbose,"\n%d\t%d\n\n",(K),(L));			\ */

#ifdef	PRINT3D
#undef	PRINT3D
#endif
#define		PRINT3D(Verbose, FormatString, Pointer, K, L, M)\
do	{							\
if(Verbose)	{						\
	int __k__, __l__, __m__;				\
if((Verbose)==stderr || (Verbose)==stdout)			\
fprintf(Verbose,"\n\n%s[%s=%d][%s=%d][%s=%d]:",			\
#Pointer,  (#K), (K), (#L), (L), (#M), (M));			\
else								\
fprintf(Verbose,"\n\n%s[%s=%d][%s=%d][%s=%d]:",			\
#Pointer,  (#K), (K), (#L), (L), (#M), (M));			\
for(	__k__=0;	__k__<(K);	__k__++)	{	\
if((Verbose)==stderr || (Verbose)==stdout)			\
fprintf(Verbose,"\n\nMatrix %d:\n",__k__+1);			\
else								\
fprintf(Verbose,"\n");						\
for(	__l__=0;	__l__<(L);	__l__++)	{	\
fprintf(Verbose,"\n");						\
for(	__m__=0;	__m__<(M);	__m__++)	{	\
fprintf(Verbose,FormatString,(Pointer)[__k__][__l__][__m__]);	\
}}}}								\
}while(0)
/* fprintf(Verbose,"\n%d\t%d\t%d\n\n",(K),(L),(M));		\ */

#ifdef	PRINT4D
#undef	PRINT4D
#endif
#define		PRINT4D(Verbose, FormatString, Pointer, K, L, M, N)\
do	{							\
if(Verbose)	{						\
	int __k__, __l__, __m__, __n__;				\
if((Verbose)==stderr || (Verbose)==stdout)			\
fprintf(Verbose,"\n\n%s[%s=%d][%s=%d][%s=%d][%s=%d]:",		\
#Pointer,  (#K), (K), (#L), (L), (#M), (M), (#N), (N));		\
else								\
fprintf(Verbose,"\n%d\t%d\t%d\t%d\n\n",(K),(L),(M), (N));	\
for(	__k__=0;	__k__<(K);	__k__++)	{	\
if((Verbose)==stderr || (Verbose)==stdout)			\
fprintf(Verbose,"\n\nBLOCK %d:",__k__+1);			\
else								\
fprintf(Verbose,"\n");						\
for(	__l__=0;	__l__<(L);	__l__++)	{	\
if((Verbose)==stderr || (Verbose)==stdout)			\
fprintf(Verbose,"\n\nMatrix %d:\n",__l__+1);			\
else								\
fprintf(Verbose,"\n");						\
for(	__m__=0;	__m__<(M);	__m__++)	{	\
fprintf(Verbose,"\n");						\
for(	__n__=0;	__n__<(N);	__n__++)	{	\
fprintf(Verbose,FormatString,(Pointer)[__k__][__l__][__m__][__n__]);\
}}}}}\
}while(0)

/*
    Title: 	SCAN1D till SCAN4D.
    Purpose:	Reading matrices from file (Verbose), in which 1D reads
                one array, 2D reads one matrix, 3D reads more matrices
		and 4D reads blocks of matrices.
		The filepointer needs to be positioned in the line below
		the file-containment-parameters
		[#blocks], [#matrices], [#rows] and #columns.
		ReadAndDetermineDimensions (in read.h) determines the
		dimensions and positions the filepointer on the right
		position.

    Created:	Mon Oct 18 11:39:07 1993
    Author: 	Patrick Timmermans, student of Adrie Dane
		<adstud1@sci.kun.nl>
*/

#ifdef	SCAN1D
#undef	SCAN1D
#endif
#define		SCAN1D(Verbose, FormatString, Pointer, K)		\
do	{								\
if(Verbose)	{							\
	int __k__;							\
for(	__k__=0;	__k__<(K);	__k__++)			\
fscanf(Verbose, FormatString, (Pointer)+__k__);			\
}									\
}while(0)

#ifdef	SCAN2D
#undef	SCAN2D
#endif
#define		SCAN2D(Verbose, FormatString, Pointer, K, L)		\
do	{								\
if(Verbose)	{							\
	int __k__, __l__;						\
for(	__k__=0;	__k__<(K);	__k__++)	{		\
for(	__l__=0;	__l__<(L);	__l__++)	{		\
fscanf(Verbose,FormatString,(Pointer)[__k__]+__l__);			\
}}}									\
}while(0)

#ifdef	SCAN3D
#undef	SCAN3D
#endif
#define		SCAN3D(Verbose, FormatString, Pointer, K, L, M)		\
do	{								\
if(Verbose)	{							\
	int __k__, __l__, __m__;					\
for(	__k__=0;	__k__<(K);	__k__++)	{		\
for(	__l__=0;	__l__<(L);	__l__++)	{		\
for(	__m__=0;	__m__<(M);	__m__++)	{		\
fscanf(Verbose,FormatString,(Pointer)[__k__][__l__]+__m__);		\
}}}}									\
}while(0)

#ifdef	SCAN4D
#undef	SCAN4D
#endif
#define		SCAN4D(Verbose, FormatString, Pointer, K, L, M, N)	\
do	{								\
if(Verbose)	{							\
	int __k__, __l__, __m__, __n__;					\
for(	__k__=0;	__k__<(K);	__k__++)	{		\
for(	__l__=0;	__l__<(L);	__l__++)	{		\
for(	__m__=0;	__m__<(M);	__m__++)	{		\
for(	__n__=0;	__n__<(N);	__n__++)	{		\
fscanf(Verbose,FormatString,(Pointer)[__k__][__l__][__m__]+__n__);	\
}}}}}									\
}while(0)

/*
  Converts normal allocated 2D Pointer[0 .. __K__-1][0 .. __L__-1]
  To NRPointer[K_low .. K_high][L_Low .. L_High] as are used in Numerical
  Recipes
  Examples:
  1.
    CALLOC2D(Pointer,Nrows,Ncolumns);
    CONVERT2D(NRPointer,Pointer,1,Nrows,1,Ncolumns);
  2.
    double	Matrix[Nrows][Ncolumns];
    CONVERT2D(NRPointer,Matrix,1,Nrows,1,Ncolumns);
  Both make NRPointer be a matrix indexed [1][1] till [Nrows][Ncolumns]
*/

#ifdef	CONVERT2D
#undef	CONVERT2D
#endif
#define		CONVERT2D(NRPointer,Pointer,K_low,K_high,L_low,L_high)	\
do	{								\
        int __i__,__j__,__K__,__L__;					\
        __K__=(K_high)-(K_low)+1;					\
        __L__=(L_high)-(L_low)+1;					\
        (NRPointer) = calloc(	(__K__) ,	sizeof(*(NRPointer)));	\
	if	(IS_NOT_ALLOCATED(NRPointer))	{			\
		if	(stderr != NULL)				\
			fprintf(stderr, "\
\a%sERROR\n\
in macro 'CALLOC1D' defined in file '%s'\n\
called in file '%s' on line %u:\n\
\tMemory allocation for dynamic 1D-array failed.\n\
\tAborting . . .\n",	__HFILE__, __HFILE__, __FILE__, __LINE__);	\
}									\
        (NRPointer) -= (K_low);						\
	for(__i__=0, __j__=(K_low); __i__<__K__; __i__++, __j__++)	\
		(NRPointer)[__j__]=(Pointer)[0]+__L__*__i__-(L_low);	\
}while	(0)



#ifdef	FREE_CONVERT2D
#undef	FREE_CONVERT2D
#endif
#define		FREE_CONVERT2D(NRPointer,K_low)				\
do	{								\
        free((NRPointer)+(K_low));					\
}	while	(0)

#endif



